#! /usr/bin/env python

from __future__ import print_function
import argparse
import os
import sys
from sqlobject.sqlbuilder import CONCAT

from m_lib.defenc import default_encoding
from m_librarian.config import get_config
from m_librarian.db import Author, Book, Extension, Genre, Language, open_db
from m_librarian.download import download
from m_librarian.search import mk_search_conditions, \
    search_authors, search_books, \
    search_extensions, search_genres, search_languages

from m_librarian.translations import translations
_ = getattr(translations, 'ugettext', None) or translations.gettext


def decode(value):
    if isinstance(value, bytes):
        return value.decode(default_encoding)
    return value


def encode(value):
    if bytes is str and not isinstance(value, bytes):
        return value.encode(default_encoding, errors='replace')
    return value


def _get_values(args, *columns):
    values = {}
    for column in columns:
        value = getattr(args, column)
        if value:
            value = decode(value)
            values[column] = value
    return values


def _guess_case_sensitivity(values):
    for value in values.values():
        if not value.islower():
            return True
    return False


def print_count(count):
    print(encode(_("Found")), ":", count)


def _search_authors(case_sensitive, search_type, args):
    if (args.surname or args.name or args.misc_name) and args.fullname:
        sys.stderr.write(
            "Cannot search by names and full name at the same time\n")
        main_parser.print_help()
        sys.exit(1)
    expressions = []
    values = _get_values(args, 'surname', 'name', 'misc_name', 'id')
    if not values:
        value = args.fullname
        if value:
            expressions.append((
                CONCAT(Author.q.surname, ' ', Author.q.name, ' ',
                       Author.q.misc_name),
                decode(value)
            ))
    if case_sensitive is None:
        case_sensitive = _guess_case_sensitivity(values)
    authors = search_authors(search_type, case_sensitive, values, expressions,
                             orderBy=('surname', 'name', 'misc_name'))
    if args.count:
        print_count(authors.count())
        return
    count = 0
    for author in authors:
        print(encode(author.fullname),
              encode((u"(%s: %d)" % (_('books'), author.count))), end=' ')
        if args.verbose >= 1:
            print("(id=%d)" % author.id, end=' ')
        print()
        count += 1
    print_count(count)


def _search_books(case_sensitive, search_type, args):
    if args.get and args.get_many:
        sys.stderr.write(
            "Cannot get one book and many books at the same time\n")
        main_parser.print_help()
        sys.exit(1)
    join_expressions = []
    values = _get_values(args, 'title', 'series', 'archive', 'file', 'id')
    if case_sensitive is None:
        test_values = values.copy()
        test_values.update(_get_values(args, 'surname', 'name', 'misc_name',
                                       'fullname', 'ext', 'gname', 'gtitle',
                                       'lang'))
        case_sensitive = _guess_case_sensitivity(test_values)
    avalues = _get_values(args, 'surname', 'name', 'misc_name', 'fullname')
    if args.aid:
        avalues['id'] = args.aid
    if avalues:
        if (args.surname or args.name or args.misc_name) and args.fullname:
            sys.stderr.write(
                "Cannot search by names and full name at the same time\n")
            main_parser.print_help()
            sys.exit(1)
        expressions = []
        join_expressions.append(Book.j.authors)
        value = args.fullname
        if value:
            expressions.append((
                CONCAT(Author.q.surname, ' ', Author.q.name, ' ',
                       Author.q.misc_name),
                decode(value)
            ))
        conditions = mk_search_conditions(
            Author, search_type, case_sensitive, avalues, expressions)
        join_expressions.extend(conditions)
    evalues = {}
    if args.ext:
        evalues['name'] = args.ext
    if args.eid:
        evalues['id'] = args.eid
    if evalues:
        join_expressions.append(Book.j.extension)
        conditions = mk_search_conditions(
            Extension, search_type, case_sensitive, evalues)
        join_expressions.extend(conditions)
    gvalues = {}
    for column in 'name', 'title':
        value = getattr(args, 'g' + column)
        if value:
            gvalues[column] = decode(value)
    if args.gid:
        gvalues['id'] = args.gid
    if gvalues:
        join_expressions.append(Book.j.genres)
        conditions = mk_search_conditions(
            Genre, search_type, case_sensitive, gvalues)
        join_expressions.extend(conditions)
    lvalues = {}
    if args.lang:
        lvalues['name'] = args.lang
    if args.lid:
        lvalues['id'] = args.lid
    if lvalues:
        join_expressions.append(Book.j.language)
        conditions = mk_search_conditions(
            Language, search_type, case_sensitive, lvalues)
        join_expressions.extend(conditions)
    books = search_books(search_type, case_sensitive, values, join_expressions,
                         orderBy=('series', 'ser_no', 'title', '-date'),
                         use_filters=args.filters)
    if args.count:
        print_count(books.count())
        return
    if args.get_many:
        books = books[:args.get_many]
    elif args.get:
        count = books.count()
        if count != 1:
            sys.stderr.write("There must be exactly 1 book for --get; "
                             "(found %d).\n" % count)
            sys.stderr.write("Use --get-many N to download more than one "
                             "book.\n")
            sys.exit(1)
    count = 0
    for book in books:
        print(encode(book.title), end=' ')
        if args.verbose >= 1:
            print("(id=%d)" % book.id, end=' ')
        print()
        if args.verbose >= 1:
            print(" ", encode(_("Author(s)")), ":", end=' ')
            for author in book.authors:
                print(encode(author.fullname), end=' ')
            print()
            print(" ", encode(_("Genre(s)")), ":", end=' ')
            for genre in book.genres:
                print((encode(genre.title or genre.name)), end=' ')
            print()
            if book.series:
                print(" ", encode(_("Series")), ":", end=' ')
                print(encode(book.series), "(%d)" % book.ser_no)

        if args.verbose >= 2:
            print(" ", encode(_("Date")), ":", book.date)
            print(" ", encode(_("Language")), ":", encode(book.language.name))

        if args.verbose >= 3:
            print(" ", encode(_("Archive")), ":", book.archive)
            print(" ", encode(_("File")), ":", book.file)
            print(" ", encode(_("Extension")), ":",
                  encode(book.extension.name))
            print(" ", encode(_("Size")), ":",
                  book.size, encode(_("bytes")))
            print(" ", encode(_("Deleted")), ":",
                  encode(_(str(book.deleted))))
        if args.get or args.get_many:
            download_to = args.download_to
            if download_to is None:
                download_to = get_config().getpath('download', 'path')
            download(book, download_to, args.path, args.format)
        count += 1
    print_count(count)


def _search_extensions(case_sensitive, search_type, args):
    values = _get_values(args, 'name', 'id')
    if case_sensitive is None:
        case_sensitive = _guess_case_sensitivity(values)
    else:
        values = {}
    extensions = search_extensions(search_type, case_sensitive, values,
                                   orderBy='name')
    if args.count:
        print_count(extensions.count())
        return
    count = 0
    for ext in extensions:
        print(encode(ext.name),
              encode((u"(%s: %d)" % (_('books'), ext.count))),
              end=' ')
        if args.verbose >= 1:
            print("(id=%d)" % ext.id, end=' ')
        print()
        count += 1
    print_count(count)


def _search_genres(case_sensitive, search_type, args):
    values = _get_values(args, 'name', 'title', 'id')
    if case_sensitive is None:
        case_sensitive = _guess_case_sensitivity(values)
    genres = search_genres(search_type, case_sensitive, values, orderBy='name')
    if args.count:
        print_count(genres.count())
        return
    count = 0
    for genre in genres:
        names = filter(None, (genre.name, genre.title))
        fullname = u' '.join(names)
        print(encode(fullname),
              encode((u"(%s: %d)" % (_('books'), genre.count))),
              end=' ')
        if args.verbose >= 1:
            print("(id=%d)" % genre.id, end=' ')
        print()
        count += 1
    print_count(count)


def _search_languages(case_sensitive, search_type, args):
    values = _get_values(args, 'name', 'id')
    if case_sensitive is None:
        case_sensitive = _guess_case_sensitivity(values)
    else:
        values = {}
    languages = search_languages(search_type, case_sensitive, values,
                                 orderBy='name')
    if args.count:
        print_count(languages.count())
        return
    count = 0
    for lang in languages:
        print(encode(lang.name),
              encode((u"(%s: %d)" % (_('books'), lang.count))),
              end=' ')
        if args.verbose >= 1:
            print("(id=%d)" % lang.id, end=' ')
        print()
        count += 1
    print_count(count)


if __name__ == '__main__':
    main_parser = argparse.ArgumentParser(description='Search')
    main_parser.add_argument('-C', '--config', help='configuration file')
    main_parser.add_argument('-D', '--database', help='database URI')
    main_parser.add_argument('-i', '--ignore-case', action='store_true',
                             help='ignore case '
                             '(default is to guess)')
    main_parser.add_argument('-I', '--case-sensitive', action='store_true',
                             help='don\'t ignore case')
    main_parser.add_argument('-t', '--start', action='store_true',
                             help='search substring at the start '
                             '(this is the default)')
    main_parser.add_argument('-s', '--substring', action='store_true',
                             help='search substring anywhere')
    main_parser.add_argument('-f', '--full', action='store_true',
                             help='match the entire string')
    main_parser.add_argument('-c', '--count', action='store_true',
                             help='count instead of output')
    main_parser.add_argument('-v', '--verbose', action='count', default=0,
                             help='output more details about books; '
                             'repeat for even more details')
    subparsers = main_parser.add_subparsers(help='Commands')

    parser = subparsers.add_parser('authors', help='Search authors')
    parser.add_argument('-s', '--surname', help='search by surname')
    parser.add_argument('-n', '--name', help='search by name')
    parser.add_argument('-m', '--misc-name', help='search by misc. name')
    parser.add_argument('--id', type=int, help='search by database id')
    parser.add_argument('fullname', nargs='?', help='search by full name')
    parser.set_defaults(func=_search_authors)

    parser = subparsers.add_parser('books', help='Search books')
    parser.add_argument('-t', '--title', help='search by title')
    parser.add_argument('-s', '--series', help='search by series')
    parser.add_argument('-a', '--archive', help='search by archive (zip file)')
    parser.add_argument('-f', '--file', help='search by file name')
    parser.add_argument('--id', type=int, help='search by database id')
    parser.add_argument('--surname', help='search by author\'s surname')
    parser.add_argument('--name', help='search by author\'s name')
    parser.add_argument('--misc-name', help='search by author\'s misc. name')
    parser.add_argument('--fullname', help='search by author\'s full name')
    parser.add_argument('--aid', type=int, help='search by author\'s id')
    parser.add_argument('-e', '--ext', help='search by file extension')
    parser.add_argument('--eid', type=int, help='search by extension\'s id')
    parser.add_argument('--gname', help='search by genre\'s name')
    parser.add_argument('--gtitle', help='search by genre\'s title')
    parser.add_argument('--gid', type=int, help='search by genre\'s id')
    parser.add_argument('-l', '--lang', help='search by language')
    parser.add_argument('--lid', type=int, help='search by language\'s id')
    parser.add_argument('--filters', action='store_true',
                        help='use filters from config')
    parser.add_argument('-P', '--path', help='path to the library archives')
    parser.add_argument('--download-to', nargs='?',
                        const=None, default=os.path.curdir,
                        help='download directory')
    parser.add_argument('-F', '--format',
                        help='download format, default is %%f')
    parser.add_argument('--get', action='store_true',
                        help='download exactly one book')
    parser.add_argument('--get-many', type=int,
                        help='download at most this many books')
    parser.set_defaults(func=_search_books)

    parser = subparsers.add_parser('ext', help='Search extensions')
    parser.add_argument('name', nargs='?', help='search by name')
    parser.add_argument('--id', type=int, help='search by database id')
    parser.set_defaults(func=_search_extensions)

    parser = subparsers.add_parser('genres', help='Search genres')
    parser.add_argument('-n', '--name', help='search by name')
    parser.add_argument('-t', '--title', help='search by title')
    parser.add_argument('--id', type=int, help='search by database id')
    parser.set_defaults(func=_search_genres)

    parser = subparsers.add_parser('lang', help='Search languages')
    parser.add_argument('name', nargs='?', help='search by name')
    parser.add_argument('--id', type=int, help='search by database id')
    parser.set_defaults(func=_search_languages)

    args = main_parser.parse_args()

    if args.config:
        get_config(args.config)  # Get and cache config file

    if args.case_sensitive:
        if args.ignore_case:
            sys.stderr.write(
                "Cannot search case sensitive and case insensitive "
                "at the same time\n")
            main_parser.print_help()
            sys.exit(1)
        else:
            case_sensitive = True
    elif args.ignore_case:
        case_sensitive = False
    else:
        case_sensitive = None  # guess case sensitivity

    if int(args.start) + int(args.substring) + int(args.full) > 1:
        sys.stderr.write(
            "Cannot search from start, substring and full "
            "at the same time\n")
        main_parser.print_help()
        sys.exit(1)
    if args.start:
        search_type = 'start'
    elif args.substring:
        search_type = 'substring'
    elif args.full:
        search_type = 'full'
    else:
        search_type = 'start'

    open_db(args.database)
    args.func(case_sensitive, search_type, args)



import sys

# Compatability definitions
PY2 = sys.version_info[0] < 3
if PY2:
    # disable flake8 checks on python 3
    string_type = basestring  # noqa
    unicode_type = unicode  # noqa
else:
    string_type = str
    unicode_type = str




import os
from m_librarian.config import get_config


def test_config():
    config_path = os.path.join(
        os.path.dirname(__file__), 'test_config.conf')
    get_config(config_path)
    ml_conf = get_config()
    assert ml_conf.get('library', 'path') == '/home/test-config'




from dbutils import setup_module, teardown_module  # noqa
from m_librarian.db import Genre
from m_librarian.glst import import_glst


def test_import_glst():
    import_glst()
    assert Genre.select().count() == 370



#! /usr/bin/env python

import argparse
import time
import webbrowser

from bottle import thread  # portable import

from m_librarian.db import open_db
import m_librarian.web.app  # noqa: F401 imported but unused
from m_librarian.web.server import run_server
from m_librarian.web.utils import get_lock, close_lock, get_open_port


def start_browser(port):
    time.sleep(1)  # A small timeout to allow the main thread to run the server
    webbrowser.open_new('http://localhost:%d/' % port)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Init')
    parser.add_argument('-p', '--port', help='HTTP server port')
    args = parser.parse_args()

    if args.port:
        port = args.port
    else:
        port = get_open_port()

    lock_file, old_port = get_lock(port)
    if lock_file:
        open_db()
        thread.start_new_thread(start_browser, (port,))
        run_server(port=port)
        close_lock(lock_file)
    else:  # Another instance of the program is being run at a different port
        start_browser(old_port)



#!/usr/bin/env python
# -*- coding: utf-8 -*-




##################################################
## DEPENDENCIES
import sys
import os
import os.path
try:
    import builtins as builtin
except ImportError:
    import __builtin__ as builtin
from os.path import getmtime, exists
import time
import types
from Cheetah.Version import MinCompatibleVersion as RequiredCheetahVersion
from Cheetah.Version import MinCompatibleVersionTuple as RequiredCheetahVersionTuple
from Cheetah.Template import Template
from Cheetah.DummyTransaction import *
from Cheetah.NameMapper import NotFound, valueForName, valueFromSearchList, valueFromFrameOrSearchList
from Cheetah.CacheRegion import CacheRegion
import Cheetah.Filters as Filters
import Cheetah.ErrorCatchers as ErrorCatchers
from Cheetah.compat import unicode

##################################################
## MODULE CONSTANTS
VFFSL=valueFromFrameOrSearchList
VFSL=valueFromSearchList
VFN=valueForName
currentTime=time.time
__CHEETAH_version__ = '3.2.6'
__CHEETAH_versionTuple__ = (3, 2, 6, 'final', 0)
__CHEETAH_genTime__ = 1608677601.931249
__CHEETAH_genTimestamp__ = 'Wed Dec 23 01:53:21 2020'
__CHEETAH_src__ = 'layout.tmpl'
__CHEETAH_srcLastModified__ = 'Wed Dec 23 01:52:26 2020'
__CHEETAH_docstring__ = 'Autogenerated by Cheetah: The Python-Powered Template Engine'

if __CHEETAH_versionTuple__ < RequiredCheetahVersionTuple:
    raise AssertionError(
      'This template was compiled with Cheetah version'
      ' %s. Templates compiled before version %s must be recompiled.'%(
         __CHEETAH_version__, RequiredCheetahVersion))

##################################################
## CLASSES

class layout(Template):

    ##################################################
    ## CHEETAH GENERATED METHODS


    def __init__(self, *args, **KWs):

        super(layout, self).__init__(*args, **KWs)
        if not self._CHEETAH__instanceInitialized:
            cheetahKWArgs = {}
            allowedKWs = 'searchList namespaces filter filtersLib errorCatcher'.split()
            for k,v in KWs.items():
                if k in allowedKWs: cheetahKWArgs[k] = v
            self._initCheetahInstance(**cheetahKWArgs)
        

    def respond(self, trans=None):



        ## CHEETAH: main method generated for this template
        if (not trans and not self._CHEETAH__isBuffering and not callable(self.transaction)):
            trans = self.transaction # is None unless self.awake() was called
        if not trans:
            trans = DummyTransaction()
            _dummyTrans = True
        else: _dummyTrans = False
        write = trans.response().write
        SL = self._CHEETAH__searchList
        _filter = self._CHEETAH__currentFilter
        
        ########################################
        ## START - generated method body
        
        write(u'''<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
   "http://www.w3.org/TR/html4/loose.dtd">
<html>
<!-- THIS PAGE IS AUTOMATICALLY GENERATED USING CheetahTemplate. DO NOT EDIT. -->
<head>
<meta HTTP-Equiv="Content-Type" content="text/html; charset=utf-8">
<title>''')
        _v = VFFSL(SL,"title",True) # u'$title' on line 9, col 8
        if _v is not None: write(_filter(_v, rawExpr=u'$title')) # from line 9, col 8.
        write(u''' \u2014 m_Librarian</title>
<meta name="author" content="Oleg Broytman">
<meta name="copyright" content="&copy; 2018 PhiloSoft Design">
<link rev="made" href="mailto:phd@phdru.name">
<meta name="generator" content="CheetahTemplate">
<meta name="MSSmartTagsPreventParsing" content="TRUE"> <!-- MUST DIE -->
<link rel="stylesheet" type="text/css" media="screen" href="/static/style.css">
</head>
<body bgcolor="#FFFFFF" text="#000000"
      link="#0000bb"  vlink="#551a8b" alink="#ff0000">
<table style="width: 100%">
<tr style="width: 100%">
  <td style="width: 50%"><p><a href="/">m_Librarian</a></p></td>
  <td style="width: 50%; text-align: right"><a href="/quit">\u0412\u044b\u0445\u043e\u0434</a></td>
</tr>
</table>
''')
        _v = VFFSL(SL,"body",True) # u'$body' on line 25, col 1
        if _v is not None: write(_filter(_v, rawExpr=u'$body')) # from line 25, col 1.
        write(u'''
</body>
</html>
''')
        
        ########################################
        ## END - generated method body
        
        return _dummyTrans and trans.response().getvalue() or ""
        
    ##################################################
    ## CHEETAH GENERATED ATTRIBUTES


    _CHEETAH__instanceInitialized = False

    _CHEETAH_version = __CHEETAH_version__

    _CHEETAH_versionTuple = __CHEETAH_versionTuple__

    _CHEETAH_genTime = __CHEETAH_genTime__

    _CHEETAH_genTimestamp = __CHEETAH_genTimestamp__

    _CHEETAH_src = __CHEETAH_src__

    _CHEETAH_srcLastModified = __CHEETAH_srcLastModified__

    title = ''

    _mainCheetahMethod_for_layout = 'respond'

## END CLASS DEFINITION

if not hasattr(layout, '_initCheetahAttributes'):
    templateAPIClass = getattr(layout,
                               '_CHEETAH_templateClass',
                               Template)
    templateAPIClass._addCheetahPlumbingCodeToClass(layout)


# CHEETAH was developed by Tavis Rudd and Mike Orr
# with code, advice and input from many other volunteers.
# For more information visit https://cheetahtemplate.org/

##################################################
## if run from command line:
if __name__ == '__main__':
    from Cheetah.TemplateCmdLineIface import CmdLineIface
    CmdLineIface(templateObj=layout()).run()





import os
import sys
from wsgiref import simple_server
from wsgiref.handlers import SimpleHandler
from wsgiref.simple_server import WSGIServer

from bottle import route, run

simple_server.ServerHandler = SimpleHandler  # Stop logging to stdout


class QuitWSGIServer(WSGIServer):
    _quit_flag = False

    def serve_forever(self):
        while not self._quit_flag:
            self.handle_request()


@route('/quit')
def quit():
    QuitWSGIServer._quit_flag = True
    return "The program has finished. Have a nice day!"


def run_server(host='localhost', port=0):
    # Start here so that bottle can find templates
    os.chdir(os.path.dirname(__file__))
    sys.path.insert(0, os.getcwd())  # To import Cheetah templates
    run(host=host, port=port, server_class=QuitWSGIServer, debug=True)



# -*- coding: utf-8 -*-

import os

from sqlobject.sqlbuilder import CONCAT
from bottle import cheetah_view, redirect, request, route, static_file

from m_librarian.config import get_config
from m_librarian.db import Author, Book
from m_librarian.download import download
from m_librarian.search import search_authors, search_books


@route('/')
@cheetah_view('index.tmpl')
def index():
    return {
        'get_config': get_config,
    }


@route('/search_authors', method='GET')
def _search_authors():
    return redirect('/search_authors/')


@route('/search_authors/', method='GET')
@cheetah_view('search_authors.tmpl')
def search_authors_get():
    return {}


def decode(value):
    if isinstance(value, bytes):
        return value.decode('utf-8')
    return value


def _guess_case_sensitivity(value):
    return not value.islower()


@route('/search_authors/', method='POST')
@cheetah_view('list_authors.tmpl')
def search_authors_post():
    value = request.forms.get('search_authors')
    if not value:
        return redirect('/search_authors/')
    value = decode(value)
    search_type = request.forms.get('search_type')
    if not search_type:
        search_type = 'start'
    case_sensitive = request.forms.get('case_sensitive')
    if case_sensitive is None:
        case_sensitive = _guess_case_sensitivity(value)
    expressions = [(
        CONCAT(Author.q.surname, ' ', Author.q.name, ' ', Author.q.misc_name),
        decode(value)
    )]
    authors = search_authors(search_type, case_sensitive, {}, expressions,
                             orderBy=('surname', 'name', 'misc_name'))
    columns = get_config().getlist('columns', 'author', ['fullname'])
    return {
        'authors': list(authors),
        'search_authors': value,
        'search_type': search_type,
        'case_sensitive': case_sensitive,
        'columns': columns,
    }


@route('/books-by-author/<aid:int>/', method='GET')
@cheetah_view('list_books.tmpl')
def books_by_author(aid):
    use_filters = get_config().getint('filters', 'use_in_books_list', 1)
    columns = get_config().getlist('columns', 'book', ['title'])
    author = Author.get(aid)
    if use_filters:
        join_expressions = []
        join_expressions.append(Book.j.authors)
        join_expressions.append(Author.q.id == aid)
        books = search_books('full', None, {}, join_expressions,
                             orderBy=('series', 'ser_no', 'title', '-date'),
                             use_filters=use_filters)
    else:
        books = Book.select(
            Book.j.authors & (Author.q.id == aid),
            orderBy=['series', 'ser_no', 'title'],
        )

    return {
        'books_by_author': {author.fullname: list(books)},
        'columns': columns,
    }


@route('/static/<filename:path>')
def send_static(filename):
    return static_file(
        filename, root=os.path.join(
            os.path.dirname(__file__),
            'static'
        )
    )


@route('/download/', method='POST')
@cheetah_view('download.tmpl')
def download_books():
    books_ids = []
    form = request.forms
    for k in form:
        if k.split('_')[-1] == 'books':
            for bid in form.getall(k):
                books_ids.append(bid)
    download_path = get_config().getpath('download', 'path')
    if books_ids:
        for bid in books_ids:
            book = Book.get(int(bid))
            download(book, download_path)
        return {
            'message': u'Книги сохранены.',
        }
    else:
        return {
            'message': u'Не выбрано книг для сохранения.',
        }


@route('/search_books', method='GET')
def _search_books():
    return redirect('/search_books/')


@route('/search_books/', method='GET')
@cheetah_view('search_books.tmpl')
def search_books_get():
    return {
        'get_config': get_config,
    }


@route('/search_books/', method='POST')
@cheetah_view('list_books.tmpl')
def search_books_post():
    value = request.forms.get('search_books')
    if not value:
        return redirect('/search_books/')
    value = decode(value)
    search_type = request.forms.get('search_type')
    if not search_type:
        search_type = 'start'
    case_sensitive = request.forms.get('case_sensitive')
    if case_sensitive is None:
        case_sensitive = _guess_case_sensitivity(value)
    use_filters = request.forms.get('use_filters')
    books = search_books(search_type, case_sensitive, {'title': value}, None,
                         orderBy=('title',), use_filters=use_filters)
    books_by_authors = {}
    for book in books:
        author = book.author1
        if author in books_by_authors:
            books_by_author = books_by_authors[author]
        else:
            books_by_author = books_by_authors[author] = []
        books_by_author.append(book)
    columns = get_config().getlist('columns', 'book', ['title'])
    return {
        'books_by_author': books_by_authors,
        'search_books': value,
        'search_type': search_type,
        'case_sensitive': case_sensitive,
        'columns': columns,
    }



#! /usr/bin/env python

from __future__ import print_function
import os
try:
    from ConfigParser import RawConfigParser, NoSectionError, NoOptionError
except ImportError:  # py3
    from configparser import RawConfigParser, NoSectionError, NoOptionError


__all__ = ['get_config']


def _find_config_dirs_posix():
    config_dirs = []
    if 'XDG_CONFIG_HOME' in os.environ:
        config_dirs.append(os.environ['XDG_CONFIG_HOME'])
    if 'XDG_CONFIG_DIRS' in os.environ:
        config_dirs.extend(os.environ['XDG_CONFIG_DIRS'].split(':'))
    home_config = os.path.expanduser('~/.config')
    if home_config not in config_dirs:
        config_dirs.append(home_config)
    return config_dirs


def find_config_dirs():
    if os.name == 'posix':
        return _find_config_dirs_posix()
    return None


def find_config_file(config_dirs=None):
    if config_dirs is None:
        config_dirs = find_config_dirs()
    for d in config_dirs:
        ml_conf_file = os.path.join(d, 'm_librarian.conf')
        if os.path.exists(ml_conf_file):
            return ml_conf_file
    else:
        return None


_ml_config = None


class ConfigWrapper(object):
    def __init__(self, config):
        self.config = config

    def get(self, section, option, default=None):
        try:
            return self.config.get(section, option)
        except (NoSectionError, NoOptionError):
            return default

    def set(self, section, option, value):
        self.config.set(section, option, value)

    def getint(self, section, option, default=0):
        try:
            return self.config.getint(section, option)
        except (NoSectionError, NoOptionError):
            return default
        # Do not catch ValueError here, it must be propagated

    def getlist(self, section, option, default=None, sep=None):
        value = self.get(section, option)
        if not value:
            if default is None:
                return []
            return default
        return value.split(sep)

    def getpath(self, section, option, default=os.path.curdir):
        path = self.get(section, option, default=default)
        return os.path.expanduser(os.path.expandvars(path))


def get_config(config_path=None):
    global _ml_config
    if _ml_config is None:
        _ml_config = RawConfigParser()
        if config_path is None:
            config_path = find_config_file()
        if config_path is not None:
            _ml_config.read(config_path)
        _ml_config = ConfigWrapper(_ml_config)
    return _ml_config


def test():
    config_dirs = find_config_dirs()
    print("Config dirs:", config_dirs)
    print("Config file:", find_config_file(config_dirs))


if __name__ == '__main__':
    test()




import gettext
import locale
import os


def get_translations(language):
    mo_filename = os.path.join(
        os.path.dirname(__file__), 'translations_dir', language + '.mo')
    if os.path.exists(mo_filename):
        mo_file = open(mo_filename, 'rb')
        translations = gettext.GNUTranslations(mo_file)
        mo_file.close()
        return translations
    return None


language = locale.getdefaultlocale()[0]
translations = None

if language:
    if language in ('ru_RU', 'Russian_Russia'):
        language = 'ru'
    translations = get_translations(language)

if translations is None:
    translations = gettext.NullTranslations()



#! /usr/bin/env python
from __future__ import print_function

import argparse
import sys

from m_librarian.config import get_config
from m_librarian.db import open_db, init_db, update_counters
from m_librarian.glst import import_glst
from m_librarian.inp import import_inpx
from m_librarian.pbar import ttyProgressBar
if ttyProgressBar:
    from m_librarian.pbar import ml_ttyProgressBar

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Import')
    parser.add_argument('-C', '--config', help='configuration file')
    parser.add_argument('-D', '--database', help='database URI')
    parser.add_argument('-P', '--no-pbar', action='store_true',
                        help='inhibit progress bar')
    parser.add_argument('inpx', nargs='+', help='INPX files to import')
    args = parser.parse_args()

    if args.config:
        get_config(args.config)  # Get and cache config file

    open_db(args.database)
    init_db()

    count_old, count_new = import_glst()
    if count_old:
        print("Imported %d genres (ignored %d existing)" % (
            count_new, count_old))
    else:
        print("Imported %d genres" % count_new)

    use_pbar = ttyProgressBar and not args.no_pbar and sys.stdout.isatty()
    for inpx in args.inpx:
        if use_pbar:
            if len(inpx) > 25:
                pbar_fname = inpx[:22] + '...'
            else:
                pbar_fname = inpx
            print(pbar_fname.ljust(20), end=': ')
            sys.stdout.flush()
            pbar = ml_ttyProgressBar()
            import_inpx(inpx, pbar_cb=pbar)
            print("Ok")
        else:
            import_inpx(inpx)
    if use_pbar:
        print("Updating counters", end=': ')
        sys.stdout.flush()
        pbar = ml_ttyProgressBar()
        update_counters(pbar_cb=pbar)
        print("Ok")
    else:
        update_counters()



#!/usr/bin/env python
# -*- coding: utf-8 -*-




##################################################
## DEPENDENCIES
import sys
import os
import os.path
try:
    import builtins as builtin
except ImportError:
    import __builtin__ as builtin
from os.path import getmtime, exists
import time
import types
from Cheetah.Version import MinCompatibleVersion as RequiredCheetahVersion
from Cheetah.Version import MinCompatibleVersionTuple as RequiredCheetahVersionTuple
from Cheetah.Template import Template
from Cheetah.DummyTransaction import *
from Cheetah.NameMapper import NotFound, valueForName, valueFromSearchList, valueFromFrameOrSearchList
from Cheetah.CacheRegion import CacheRegion
import Cheetah.Filters as Filters
import Cheetah.ErrorCatchers as ErrorCatchers
from Cheetah.compat import unicode
import cgi
from m_librarian.translations import translations
from views.layout import layout

##################################################
## MODULE CONSTANTS
VFFSL=valueFromFrameOrSearchList
VFSL=valueFromSearchList
VFN=valueForName
currentTime=time.time
__CHEETAH_version__ = '3.2.6'
__CHEETAH_versionTuple__ = (3, 2, 6, 'final', 0)
__CHEETAH_genTime__ = 1671305495.844598
__CHEETAH_genTimestamp__ = 'Sat Dec 17 22:31:35 2022'
__CHEETAH_src__ = 'list_books.tmpl'
__CHEETAH_srcLastModified__ = 'Sat Dec 17 22:31:33 2022'
__CHEETAH_docstring__ = 'Autogenerated by Cheetah: The Python-Powered Template Engine'

if __CHEETAH_versionTuple__ < RequiredCheetahVersionTuple:
    raise AssertionError(
      'This template was compiled with Cheetah version'
      ' %s. Templates compiled before version %s must be recompiled.'%(
         __CHEETAH_version__, RequiredCheetahVersion))

##################################################
## CLASSES

class list_books(layout):

    ##################################################
    ## CHEETAH GENERATED METHODS


    def __init__(self, *args, **KWs):

        super(list_books, self).__init__(*args, **KWs)
        if not self._CHEETAH__instanceInitialized:
            cheetahKWArgs = {}
            allowedKWs = 'searchList namespaces filter filtersLib errorCatcher'.split()
            for k,v in KWs.items():
                if k in allowedKWs: cheetahKWArgs[k] = v
            self._initCheetahInstance(**cheetahKWArgs)
        

    def body(self, **KWS):



        ## CHEETAH: generated from #def body at line 6, col 1.
        trans = KWS.get("trans")
        if (not trans and not self._CHEETAH__isBuffering and not callable(self.transaction)):
            trans = self.transaction # is None unless self.awake() was called
        if not trans:
            trans = DummyTransaction()
            _dummyTrans = True
        else: _dummyTrans = False
        write = trans.response().write
        SL = self._CHEETAH__searchList
        _filter = self._CHEETAH__currentFilter
        
        ########################################
        ## START - generated method body
        
        write(u'''<h1>''')
        _v = VFFSL(SL,"title",True) # u'$title' on line 7, col 5
        if _v is not None: write(_filter(_v, rawExpr=u'$title')) # from line 7, col 5.
        write(u'''</h1>

<script type="text/javascript" language="Javascript">
<!--
function toggleBooks(selector, value) {
  cboxes = document.querySelectorAll(selector);
  for (var i = 0; i < cboxes.length; i++) {
    cboxes[i].checked = value;
  }
}

function toggleAll(value) {
  toggleBooks(\'input[type=checkbox]\', value);
}

function toggleAuthor(name, value) {
  toggleBooks(\'input[type=checkbox][name^=\' + name + \']\', value);
}

function toggleSeries(name, value) {
  toggleBooks(\'input[type=checkbox][name^=\' + name + \'_book]\', value);
}
// -->
</script>

''')
        if VFFSL(SL,"books_by_author",True): # generated from line 32, col 1
            write(u'''  <form action="/download/" method="POST" style="height: 80%">
  <table style="width: 100%; height: 90%">
  <tr>
  <td><input type=checkbox name="_toggleAll"
  onClick="toggleAll(this.checked)"></td>
  ''')
            if False:
                _
            _ = VFFSL(SL,"getattr",False)(VFFSL(SL,"translations",True), 'ugettext', None) or VFFSL(SL,"translations.gettext",True)
            for column in VFFSL(SL,"columns",True): # generated from line 39, col 3
                write(u'''    <td style="text-align: center">''')
                if False:
                    _(VFFSL(SL,"column",True))
                _v = VFN(VFFSL(SL,"cgi",True),"escape",False)(VFFSL(SL,"_",False)(VFFSL(SL,"column",True))) # u'$cgi.escape($_($column))' on line 40, col 36
                if _v is not None: write(_filter(_v, rawExpr=u'$cgi.escape($_($column))')) # from line 40, col 36.
                write(u'''</td>
''')
            write(u'''  </tr>
''')
            columns1 = VFFSL(SL,"len",False)(VFFSL(SL,"columns",True))+1
            for anum, author in VFFSL(SL,"enumerate",False)(VFFSL(SL,"sorted",False)(VFFSL(SL,"books_by_author",True))): # generated from line 44, col 3
                series = None
                snum = 0
                books = VFFSL(SL,"books_by_author",True)[VFFSL(SL,"author",True)]
                write(u'''  <tr>
  <td><input type=checkbox name="a''')
                _v = VFFSL(SL,"anum",True)+1 # u'${anum+1}' on line 49, col 35
                if _v is not None: write(_filter(_v, rawExpr=u'${anum+1}')) # from line 49, col 35.
                write(u'''_"
  onClick="toggleAuthor(this.name, this.checked)"></td>
  <td colspan="''')
                _v = VFFSL(SL,"columns1",True)-1 # u'${columns1-1}' on line 51, col 16
                if _v is not None: write(_filter(_v, rawExpr=u'${columns1-1}')) # from line 51, col 16.
                write(u'''"><b>''')
                _v = VFFSL(SL,"author",True) # u'$author' on line 51, col 34
                if _v is not None: write(_filter(_v, rawExpr=u'$author')) # from line 51, col 34.
                write(u'''</b></td>
  </tr>
''')
                for book in VFFSL(SL,"books",True): # generated from line 53, col 5
                    if VFFSL(SL,"book.series",True) != VFFSL(SL,"series",True): # generated from line 54, col 5
                        series = VFFSL(SL,"book.series",True)
                        snum += 1
                        write(u'''  <tr>
  <td><input type=checkbox name="a''')
                        _v = VFFSL(SL,"anum",True)+1 # u'${anum+1}' on line 58, col 35
                        if _v is not None: write(_filter(_v, rawExpr=u'${anum+1}')) # from line 58, col 35.
                        write(u'''_s''')
                        _v = VFFSL(SL,"snum",True) # u'${snum}' on line 58, col 46
                        if _v is not None: write(_filter(_v, rawExpr=u'${snum}')) # from line 58, col 46.
                        write(u'''"
  onClick="toggleSeries(this.name, this.checked)"></td>
  <td colspan="''')
                        _v = VFFSL(SL,"columns1",True)-1 # u'${columns1-1}' on line 60, col 16
                        if _v is not None: write(_filter(_v, rawExpr=u'${columns1-1}')) # from line 60, col 16.
                        write(u'''"><b>
  ''')
                        _v = VFFSL(SL,"author",True) # u'$author' on line 61, col 3
                        if _v is not None: write(_filter(_v, rawExpr=u'$author')) # from line 61, col 3.
                        write(u''' \u2014 <i>
''')
                        if VFFSL(SL,"book.series",True): # generated from line 62, col 3
                            _v = VFN(VFFSL(SL,"cgi",True),"escape",False)(VFFSL(SL,"series",True), 1) # u'$cgi.escape($series, 1)' on line 63, col 1
                            if _v is not None: write(_filter(_v, rawExpr=u'$cgi.escape($series, 1)')) # from line 63, col 1.
                        else: # generated from line 64, col 3
                            write(u'''\u0412\u043d\u0435 \u0441\u0435\u0440\u0438\u0439''')
                        write(u'''  </i></b></td>
  </tr>
''')
                    write(u'''  <tr>
  <td><input type=checkbox name="a''')
                    _v = VFFSL(SL,"anum",True)+1 # u'${anum+1}' on line 71, col 35
                    if _v is not None: write(_filter(_v, rawExpr=u'${anum+1}')) # from line 71, col 35.
                    write(u'''_s''')
                    _v = VFFSL(SL,"snum",True) # u'${snum}' on line 71, col 46
                    if _v is not None: write(_filter(_v, rawExpr=u'${snum}')) # from line 71, col 46.
                    write(u'''_books" value="''')
                    _v = VFFSL(SL,"book.id",True) # u'$book.id' on line 71, col 68
                    if _v is not None: write(_filter(_v, rawExpr=u'$book.id')) # from line 71, col 68.
                    write(u'''"></td>
''')
                    for column in VFFSL(SL,"columns",True): # generated from line 72, col 3
                        if VFFSL(SL,"column",True) in ('ser_no', 'size'): # generated from line 73, col 3
                            style = ' style="text-align: right; width: 5ex"'
                        else: # generated from line 75, col 3
                            style = ''
                        write(u'''  <td''')
                        _v = VFFSL(SL,"style",True) # u'$style' on line 78, col 6
                        if _v is not None: write(_filter(_v, rawExpr=u'$style')) # from line 78, col 6.
                        write(u'''>''')
                        _v = VFN(VFFSL(SL,"cgi",True),"escape",False)(unicode(VFFSL(SL,"getattr",False)(VFFSL(SL,"book",True), VFFSL(SL,"column",True)) or '')) # u"$cgi.escape(unicode($getattr($book, $column) or ''))" on line 78, col 13
                        if _v is not None: write(_filter(_v, rawExpr=u"$cgi.escape(unicode($getattr($book, $column) or ''))")) # from line 78, col 13.
                        write(u'''</td>
''')
                    write(u'''  </tr>
''')
            write(u'''  <tr>
  <td colspan="''')
            _v = VFFSL(SL,"columns1",True) # u'$columns1' on line 84, col 16
            if _v is not None: write(_filter(_v, rawExpr=u'$columns1')) # from line 84, col 16.
            write(u'''" style="text-align: center"><input type="submit" value="\u0421\u043a\u0430\u0447\u0430\u0442\u044c"></td>
  </tr>
  </table>
  </form>
''')
        else: # generated from line 88, col 1
            write(u'''  <p>\u041d\u0435 \u043d\u0430\u0439\u0434\u0435\u043d\u043e \u043d\u0438 \u043e\u0434\u043d\u043e\u0439 \u043a\u043d\u0438\u0433\u0438!</p>
''')
        
        ########################################
        ## END - generated method body
        
        return _dummyTrans and trans.response().getvalue() or ""
        

    def writeBody(self, **KWS):



        ## CHEETAH: main method generated for this template
        trans = KWS.get("trans")
        if (not trans and not self._CHEETAH__isBuffering and not callable(self.transaction)):
            trans = self.transaction # is None unless self.awake() was called
        if not trans:
            trans = DummyTransaction()
            _dummyTrans = True
        else: _dummyTrans = False
        write = trans.response().write
        SL = self._CHEETAH__searchList
        _filter = self._CHEETAH__currentFilter
        
        ########################################
        ## START - generated method body
        
        
        ########################################
        ## END - generated method body
        
        return _dummyTrans and trans.response().getvalue() or ""
        
    ##################################################
    ## CHEETAH GENERATED ATTRIBUTES


    _CHEETAH__instanceInitialized = False

    _CHEETAH_version = __CHEETAH_version__

    _CHEETAH_versionTuple = __CHEETAH_versionTuple__

    _CHEETAH_genTime = __CHEETAH_genTime__

    _CHEETAH_genTimestamp = __CHEETAH_genTimestamp__

    _CHEETAH_src = __CHEETAH_src__

    _CHEETAH_srcLastModified = __CHEETAH_srcLastModified__

    title = 'Список книг'

    _mainCheetahMethod_for_list_books = 'writeBody'

## END CLASS DEFINITION

if not hasattr(list_books, '_initCheetahAttributes'):
    templateAPIClass = getattr(list_books,
                               '_CHEETAH_templateClass',
                               Template)
    templateAPIClass._addCheetahPlumbingCodeToClass(list_books)


# CHEETAH was developed by Tavis Rudd and Mike Orr
# with code, advice and input from many other volunteers.
# For more information visit https://cheetahtemplate.org/

##################################################
## if run from command line:
if __name__ == '__main__':
    from Cheetah.TemplateCmdLineIface import CmdLineIface
    CmdLineIface(templateObj=list_books()).run()





# coding: utf-8

from m_librarian.translations import get_translations

translations = get_translations('ru')
_ = getattr(translations, 'ugettext', None) or translations.gettext


def test_translations():
    assert _('xxx') == u'xxx'
    assert _('File') == u'Файл'



#! /usr/bin/env python

from __future__ import print_function
import os
from sqlobject import SQLObject, StringCol, UnicodeCol, IntCol, BoolCol, \
    ForeignKey, DateCol, DatabaseIndex, RelatedJoin, \
    connectionForURI, sqlhub, SQLObjectNotFound, dberrors
from .compat import string_type
from .config import get_config

__all__ = ['Author', 'Book', 'Extension', 'Genre', 'Language',
           'AuthorBook', 'BookGenre', 'open_db', 'init_db',
           'insert_name', 'insert_author', 'update_counters',
           ]


class Author(SQLObject):
    surname = UnicodeCol(notNull=True)
    name = UnicodeCol(notNull=True)
    misc_name = UnicodeCol(notNull=True)
    count = IntCol(notNull=True)
    books = RelatedJoin('Book', otherColumn='book_id',
                        intermediateTable='author_book',
                        createRelatedTable=False)

    @property
    def fullname(self):
        fullnames = filter(None, (self.surname, self.name, self.misc_name))
        return ' '.join(fullnames)

    full_name_idx = DatabaseIndex(surname, name, misc_name, unique=True)
    count_idx = DatabaseIndex(count)


class AuthorBook(SQLObject):
    class sqlmeta:
        table = "author_book"
    author = ForeignKey('Author', notNull=True, cascade=True)
    book = ForeignKey('Book', notNull=True, cascade=True)
    author_idx = DatabaseIndex(author)
    book_idx = DatabaseIndex(book)
    main_idx = DatabaseIndex(author, book, unique=True)


class Book(SQLObject):
    authors = RelatedJoin('Author',
                          intermediateTable='author_book',
                          createRelatedTable=False)
    genres = RelatedJoin('Genre',
                         intermediateTable='book_genre',
                         createRelatedTable=False)
    title = UnicodeCol(notNull=True)
    series = UnicodeCol(notNull=True)
    ser_no = IntCol()
    archive = StringCol(notNull=True)
    file = UnicodeCol(notNull=True)
    size = IntCol(notNull=True)
    lib_id = StringCol(notNull=True)
    deleted = BoolCol(notNull=True)
    extension = ForeignKey('Extension', notNull=True)
    date = DateCol(notNull=True)
    language = ForeignKey('Language')
    title_idx = DatabaseIndex(title)
    series_idx = DatabaseIndex(series)
    ser_no_idx = DatabaseIndex(ser_no)
    archive_idx = DatabaseIndex(archive)
    archive_file_idx = DatabaseIndex(archive, file, unique=True)
    file_idx = DatabaseIndex(file)
    size_idx = DatabaseIndex(size)
    deleted_idx = DatabaseIndex(deleted)
    extension_idx = DatabaseIndex(extension)
    date_idx = DatabaseIndex(date)
    language_idx = DatabaseIndex(language)

    @property
    def author1(self):
        return self.authors[0].fullname

    @property
    def author_list(self):
        return u', '.join([a.fullname for a in self.authors])

    @property
    def genre1name(self):
        return self.genres[0].name

    @property
    def genre1title(self):
        return self.genres[0].title

    @property
    def genre_name_list(self):
        return u', '.join([g.name for g in self.genres])

    @property
    def genre_title_list(self):
        return u', '.join([g.title for g in self.genres])

    @property
    def ext(self):
        return self.extension.name

    @property
    def lang(self):
        return self.language.name


class BookGenre(SQLObject):
    class sqlmeta:
        table = "book_genre"
    book = ForeignKey('Book', notNull=True, cascade=True)
    genre = ForeignKey('Genre', notNull=True, cascade=True)
    book_idx = DatabaseIndex(book)
    genre_idx = DatabaseIndex(genre)
    main_idx = DatabaseIndex(book, genre, unique=True)


class Extension(SQLObject):
    name = UnicodeCol(notNull=True, unique=True)
    count = IntCol(notNull=True)
    count_idx = DatabaseIndex(count)


class Genre(SQLObject):
    name = UnicodeCol(notNull=True, unique=True)
    title = UnicodeCol(notNull=True)
    count = IntCol(notNull=True)
    books = RelatedJoin('Book', otherColumn='book_id',
                        intermediateTable='book_genre',
                        createRelatedTable=False)
    title_idx = DatabaseIndex(title)
    count_idx = DatabaseIndex(count)


class Language(SQLObject):
    name = UnicodeCol(notNull=True, unique=True)
    count = IntCol(notNull=True)
    count_idx = DatabaseIndex(count)


def _find_sqlite_db_dirs_posix():
    db_dirs = []
    if 'XDG_CACHE_HOME' in os.environ:
        db_dirs.append(os.environ['XDG_CACHE_HOME'])
    home_cache = os.path.expanduser('~/.cache')
    if home_cache not in db_dirs:
        db_dirs.append(home_cache)
    return db_dirs


def find_sqlite_db_dirs():
    if os.name == 'posix':
        return _find_sqlite_db_dirs_posix()
    raise OSError("Unknow OS")


def find_sqlite_dburi(db_dirs=None):
    if db_dirs is None:
        db_dirs = find_sqlite_db_dirs()
    for d in db_dirs:
        db_file = os.path.join(d, 'm_librarian.sqlite')
        if os.path.exists(db_file):
            break
    else:
        # octal; -rw-------;
        # make the database file/directory readable/writeable only by the user
        os.umask(0o66)
        db_dir = db_dirs[0]
        try:
            os.makedirs(db_dir)
        except OSError:  # Perhaps already exists
            pass
        db_file = os.path.join(db_dir, 'm_librarian.sqlite')

    return 'sqlite://%s' % db_file.replace(os.sep, '/')


def open_db(db_uri=None):
    if db_uri is None:
        db_uri = get_config().get('database', 'URI') or find_sqlite_dburi()

    if '://' not in db_uri:
        db_uri = 'sqlite://' + os.path.abspath(db_uri).replace(os.sep, '/')

    sqlhub.processConnection = connection = connectionForURI(db_uri)

    if connection.dbName == 'sqlite':
        def lower(s):
            if isinstance(s, string_type):
                return s.lower()
            return s

        sqlite = connection.module

        class MLConnection(sqlite.Connection):
            def __init__(self, *args, **kwargs):
                super(MLConnection, self).__init__(*args, **kwargs)
                self.create_function('lower', 1, lower)

        # This hack must be done at the very beginning, before the first query
        connection._connOptions['factory'] = MLConnection

        # Speedup SQLite connection
        connection.query("PRAGMA synchronous=OFF")
        connection.query("PRAGMA count_changes=OFF")
        connection.query("PRAGMA journal_mode=MEMORY")
        connection.query("PRAGMA temp_store=MEMORY")


def init_db():
    try:
        Book.select()[0]
    except IndexError:  # Table exists but is empty
        return
    except dberrors.Error:
        for table in Author, Extension, Genre, Language, Book, \
                AuthorBook, BookGenre:
            table.createTable()
    else:
        return


def insert_name(table, name, **kw):
    try:
        return table.byName(name)
    except SQLObjectNotFound:
        return table(name=name, count=0, **kw)


def insert_author(surname, name, misc_name):
    try:
        return Author.full_name_idx.get(
            surname=surname, name=name, misc_name=misc_name)
    except SQLObjectNotFound:
        return Author(surname=surname, name=name, misc_name=misc_name, count=0)


def update_counters(pbar_cb=None):
    if pbar_cb:
        count = 0
        for table in Author, Extension, Genre, Language:
            count += table.select().count()
        pbar_cb.set_max(count)

    if pbar_cb:
        count = 0

    for author in Author.select():
        author.count = AuthorBook.select(AuthorBook.q.author == author).count()
        if pbar_cb:
            count += 1
            pbar_cb.display(count)

    for ext in Extension.select():
        ext.count = Book.select(Book.q.extension == ext.id).count()
        if pbar_cb:
            count += 1
            pbar_cb.display(count)

    for genre in Genre.select():
        genre.count = BookGenre.select(BookGenre.q.genre == genre).count()
        if pbar_cb:
            count += 1
            pbar_cb.display(count)

    for language in Language.select():
        language.count = Book.select(Book.q.language == language.id).count()
        if pbar_cb:
            count += 1
            pbar_cb.display(count)

    if pbar_cb:
        pbar_cb.close()


def test():
    db_dirs = find_sqlite_db_dirs()
    print("DB dirs:", db_dirs)
    print("DB URI:", find_sqlite_dburi())


if __name__ == '__main__':
    test()




import os
from m_librarian import download
from m_librarian.config import get_config


def test_compile_format():
    config_path = os.path.join(
        os.path.dirname(__file__), 'test_config.conf')
    get_config(config_path)
    ml_conf = get_config()
    ml_conf.set('download', 'format', '%a/%s/%n %t')
    download._compile_format()
    assert download.compiled_format == \
        u'%(author1)s/%(series)s/%(ser_no)d %(title)s'



#! /usr/bin/env python

from __future__ import print_function
import codecs
from glob import glob
import os
from sqlobject import sqlhub, SQLObjectNotFound
from .db import Genre

__all__ = ['import_glst']


def parse_glst_file(glst_filename):
    glst_file = codecs.open(glst_filename, 'r', 'utf-8')
    try:
        for line in glst_file:
            line = line.strip()
            if not line:
                continue
            if line[0] == '#':
                continue
            parts = line.split(None, 1)
            try:
                name, title = parts[1].split(';', 1)
            except (IndexError, ValueError):
                continue
            yield name, title
    finally:
        glst_file.close()


def import_glst_file(glst_filename):
    old = new = 0
    for name, title in parse_glst_file(glst_filename):
        try:
            Genre.byName(name)
        except SQLObjectNotFound:
            Genre(name=name, title=title, count=0)
            new += 1
        else:
            old += 1
    return old, new


def _import_glst():
    ml_dir = os.path.dirname(__file__)
    count_old = count_new = 0
    for glst_file in glob(os.path.join(ml_dir, 'glst', '*.glst')):
        _count_old, _count_new = import_glst_file(glst_file)
        count_old += _count_old
        count_new += _count_new
    connection = sqlhub.processConnection
    if connection.dbName == 'postgres':
        connection.query("VACUUM %s" % Genre.sqlmeta.table)
    return count_old, count_new


def import_glst():
    count_old, count_new = sqlhub.doInTransaction(_import_glst)
    connection = sqlhub.processConnection
    if connection.dbName == 'sqlite':
        connection.query("VACUUM")
    return count_old, count_new


def test():
    ml_dir = os.path.dirname(__file__)
    for i, (name, title) in enumerate(parse_glst_file(
        os.path.join(ml_dir, 'glst', 'genres_fb2_flibusta.glst')
    )):
        if i < 5:
            print(name, title)
        else:
            break


if __name__ == '__main__':
    test()




from pytest import raises
from dbutils import setup_module, teardown_module  # noqa
from dbutils import load_inpx
from m_librarian.db import Author, Book


def test_import_bad_inpx():
    raises(ValueError, load_inpx, 'bad.inpx')


def test_import_inpx():
    load_inpx('test.inpx')
    assert Author.select().count() == 5
    assert Book.select().count() == 5



#! /usr/bin/env python

import argparse
from m_librarian.config import get_config
from m_librarian.db import open_db, init_db
from m_librarian.glst import import_glst

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Init')
    parser.add_argument('-C', '--config', help='configuration file')
    parser.add_argument('-D', '--database', help='database URI')
    args = parser.parse_args()

    if args.config:
        get_config(args.config)  # Get and cache config file

    open_db(args.database)
    init_db()
    count_old, count_new = import_glst()
    if count_old:
        print("Imported %d genres (ignored %d existing)" % (
            count_new, count_old))
    else:
        print("Imported %d genres" % count_new)




import os
from zipfile import ZipFile
from sqlobject import sqlhub
from sqlobject.sqlbuilder import Select
from .db import Author, Book, Extension, Genre, Language, \
    insert_name, insert_author

__all__ = ['import_inpx']


EOT = chr(4)  # INP field separator


def split_line(line):
    parts = line.strip().split(EOT)
    _l = len(parts)
    if _l < 11:
        raise ValueError('Unknown INP structure: "%s"' % line)
    archive = None
    if _l == 11:  # Standard structure
        parts.append(None)  # Emulate lang
    elif _l == 15:  # New structure
        parts = parts[:12]
    elif _l == 17:  # Very new structure
        archive = parts[12]
        language = parts[13]
        parts = parts[:11] + [language]
    else:  # New structure
        raise ValueError('Unknown INP structure: "%s"' % line)
    return archive, parts


def import_inp_line(archive, parts):
    authors, genres, title, series, ser_no, file, size, lib_id, deleted, \
        extension, date, language = parts
    try:
        ser_no = int(ser_no)
    except ValueError:
        ser_no = None
    size = int(size)
    deleted = deleted == '1'
    extension_row = insert_name(Extension, extension)
    language_row = insert_name(Language, language)
    book = Book(title=title, series=series, ser_no=ser_no,
                archive=archive, file=file, size=size,
                lib_id=lib_id, deleted=deleted,
                extension=extension_row, date=date,
                language=language_row)
    authors = authors.split(':')
    seen_authors = set()
    for author in authors:
        if author:
            if author in seen_authors:
                continue
            seen_authors.add(author)
            alist = author.split(',', 2)
            surname = alist[0]
            if len(alist) > 1:
                name = alist[1]
                if len(alist) == 3:
                    misc_name = alist[2]
                else:
                    misc_name = ''
            else:
                name = misc_name = ''
            author_row = insert_author(surname, name, misc_name)
            book.addAuthor(author_row)
    for genre in genres.split(':'):
        if genre:
            genre_row = insert_name(Genre, genre, title=genre)
            book.addGenre(genre_row)


def tounicode(s):
    if isinstance(s, bytes):
        return s.decode('utf-8')
    else:
        return s


def import_inp(archive, inp):
    archives = set()
    files = set()
    connection = sqlhub.processConnection
    for file, in connection.queryAll(connection.sqlrepr(
            Select(Book.q.file, Book.q.archive == archive))):
        files.add((archive, tounicode(file)))
    for line in inp:
        line = line.decode('utf-8')
        _archive, parts = split_line(line)
        if _archive and (_archive not in archives):
            archives.add(_archive)
            for file, in connection.queryAll(connection.sqlrepr(
                    Select(Book.q.file, Book.q.archive == _archive))):
                files.add((_archive, tounicode(file)))
        file = parts[5]
        if (_archive or archive, file) not in files:
            files.add((_archive or archive, file))
            import_inp_line(_archive or archive, parts)


def import_inpx(path, pbar_cb=None):
    inpx = ZipFile(path)
    if pbar_cb:
        inp_count = 0
        for name in inpx.namelist():
            ext = os.path.splitext(name)[1]
            if ext == '.inp':
                inp_count += 1
        pbar_cb.set_max(inp_count)
    inp_count = 0
    for name in inpx.namelist():
        archive, ext = os.path.splitext(name)
        if ext != '.inp':
            continue
        if pbar_cb:
            inp_count += 1
            pbar_cb.display(inp_count)
        inp = inpx.open(name)
        sqlhub.doInTransaction(import_inp, archive + '.zip', inp)
        inp.close()
    connection = sqlhub.processConnection
    if connection.dbName == 'postgres':
        for table in Author, Book, Extension, Genre, Language:
            connection.query("VACUUM %s" % table.sqlmeta.table)
    elif connection.dbName == 'sqlite':
        connection.query("VACUUM")
    if pbar_cb:
        pbar_cb.close()



#!/usr/bin/env python

from os.path import abspath, dirname, join
from setuptools import setup
import sys

versionpath = join(abspath(dirname(__file__)), 'm_librarian', '__version__.py')
m_librarian_version = {}

if sys.version_info[:2] == (2, 7):
    execfile(versionpath, m_librarian_version)  # noqa: F821 'execfile' Py3

elif sys.version_info >= (3, 4):
    exec(open(versionpath, 'r').read(), m_librarian_version)

else:
    raise ImportError("m_librarian requires Python 2.7 or 3.4+")

setup(
    name='m_librarian',
    version=m_librarian_version['__version__'],
    description='m_Librarian for LibRusEc/Flibusta libraries',
    long_description=open('README.rst', 'r').read(),
    long_description_content_type="text/x-rst",
    author='Oleg Broytman',
    author_email='phd@phdru.name',
    url='https://phdru.name/Software/Python/m_librarian/',
    project_urls={
        'Homepage': 'https://phdru.name/Software/Python/m_librarian/',
        'Download': 'https://pypi.org/project/m_librarian/%s/'
        % m_librarian_version['__version__'],
        'Documentation':
            'https://phdru.name/Software/Python/m_librarian/docs/',
        'Russian docs':
            'https://phdru.name/Software/Python/m_librarian/docs/ru/',
        'Git repo': 'https://git.phdru.name/m_librarian.git/',
        'Github repo': 'https://github.com/phdru/m_librarian',
        'Issue tracker': 'https://github.com/phdru/m_librarian/issues',
    },
    license='GPL',
    keywords=['books', 'library', 'Flibusta', 'LibRusEc', 'lib.rus.ec'],
    platforms="Any",
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Console',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Natural Language :: English',
        'Natural Language :: Russian',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
    ],
    packages=['m_librarian', 'm_librarian.web'],
    package_data={'m_librarian': [
        'glst/*.txt', 'glst/genres_*.glst',
        'translations_dir/*.mo', 'translations_dir/*.po',
        'web/static/style.css', 'web/views/*.py', 'web/views/*.tmpl',
        ]
    },
    scripts=['scripts/ml-import.py', 'scripts/ml-initdb.py',
             'scripts/ml-search.py', 'scripts/ml-web.py'],
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*',
    install_requires=[
        'SQLObject>=2.2.1; python_version=="2.7"',
        'SQLObject>=3.0.0; python_version>="3.4"',
        'm_lib.defenc>=1.0',
    ],
    extras_require={
        'pbar': ['m_lib>=3.1'],
        'web': ['bottle', 'Cheetah3'],
    },
)



# coding: utf-8

from dbutils import setup_module, teardown_module  # noqa
from dbutils import load_inpx
from m_librarian.db import Author, Book
from m_librarian.search import mk_search_conditions, \
    search_authors, search_books


def test_search_authors():
    load_inpx('test.inpx')
    assert search_authors('full', True, {'surname': u'Друг'}).count() == 1
    assert search_authors('start', True, {'surname': u'Друг'}).count() == 2
    assert search_authors('substring', True, {'surname': u'Друг'}).count() == 2
    assert search_authors(
        'substring', False, {'surname': u'друг'}).count() == 3

    join_expressions = []
    join_expressions.append(Book.j.authors)
    conditions = mk_search_conditions(
        Author, 'start', False, {'surname': u'друг'})
    join_expressions.extend(conditions)
    assert search_books(
        'start', False, {'title': u'тест'}, join_expressions).count() == 2




try:
    from m_lib.pbar.tty_pbar import ttyProgressBar
except ImportError:
    ttyProgressBar = None

if ttyProgressBar:
    class ml_ttyProgressBar(object):
        def __init__(self, width=20):
            self.max = None
            self.pbar = None
            self.width = width

        def set_max(self, max_value):
            self.max = max_value
            self.pbar = ttyProgressBar(0, max_value, width1=self.width)

        def display(self, value):
            if self.pbar:
                self.pbar.display(value)

        def close(self):
            if self.pbar:
                self.pbar.erase()
                self.pbar = None



#! /usr/bin/env python

from __future__ import print_function
import os
from time import mktime
from shutil import copyfileobj
from zipfile import ZipFile
from .config import get_config

__all__ = ['download']


format = '%f'
compile_format = True
compiled_format = '%(file)s'


def _compile_format():
    global format, compile_format, compiled_format
    if not compile_format:
        return
    compile_format = False
    format = get_config().get('download', 'format')
    if not format:
        return
    got_percent = False
    compiled = []
    for c in format:
        if c == '%':
            if got_percent:
                got_percent = False
                compiled.append('%')
            else:
                got_percent = True
        else:
            if got_percent:
                got_percent = False
                if c == 'a':
                    new_format = u'%(author1)s'
                elif c == 'A':
                    new_format = u'%(authors)s'
                elif c == 'e':
                    new_format = u'%(extension)s'
                elif c == 'f':
                    new_format = u'%(file)s'
                elif c == 'G':
                    new_format = u'%(gname)s'
                elif c == 'g':
                    new_format = u'%(gtitle)s'
                elif c == 'J':
                    new_format = u'%(gname_list)s'
                elif c == 'j':
                    new_format = u'%(gtitle_list)s'
                elif c == 'l':
                    new_format = u'%(language)s'
                elif c == 'n':
                    new_format = u'%(ser_no)d'
                elif c == 's':
                    new_format = u'%(series)s'
                elif c == 't':
                    new_format = u'%(title)s'
                else:
                    raise ValueError('Bad format specifier "%%%c"' % c)
                compiled.append(new_format)
            else:
                compiled.append(c)
    compiled_format = ''.join(compiled)


_library_path = None


def download(book, dest_path=None, lib_path=None, a_format=None):
    if lib_path is None:
        global _library_path
        if _library_path is None:
            _library_path = get_config().getpath('library', 'path')
        lib_path = _library_path

    global format, compile_format, compiled_format
    if a_format:
        compile_format = True
        format = a_format
    _compile_format()
    if compiled_format[-1] in ('\0', '\\', '/'):
        raise ValueError('Bad format: "%s"' % compiled_format)
    bdict = {}
    bdict['author1'] = book.author1
    bdict['authors'] = book.author_list
    bdict['extension'] = book.extension.name
    bdict['file'] = book.file
    genre = book.genres[0]
    bdict['gname'] = genre.name
    bdict['gtitle'] = genre.title
    bdict['gname_list'] = book.genre_name_list
    bdict['gtitle_list'] = book.genre_title_list
    bdict['language'] = book.language.name
    bdict['ser_no'] = book.ser_no or 0
    bdict['series'] = book.series
    bdict['title'] = book.title
    if '%(extension)s' not in compiled_format:
        compiled_format += '.%(extension)s'
    filename = compiled_format % bdict
    full_path = os.path.join(dest_path, filename)
    try:
        os.makedirs(os.path.dirname(full_path))
    except OSError:
        pass  # Already exists
    zf = ZipFile(os.path.join(lib_path, book.archive),  'r')
    infile = zf.open('%s.%s' % (book.file, book.extension.name))
    outfile = open(full_path, 'wb')
    copyfileobj(infile, outfile)
    outfile.close()
    infile.close()
    zf.close()
    dt = mktime(book.date.timetuple())
    os.utime(full_path, (dt, dt))


def test():
    _compile_format()
    print(compiled_format)


if __name__ == '__main__':
    test()




import os
from sqlobject.tests.dbtest import getConnection
from m_librarian.db import open_db, init_db
from m_librarian.inp import import_inpx

__all__ = ['setup_module', 'teardown_module', 'load_inpx']


def setup_module():
    connection = getConnection()
    if connection.dbName == 'sqlite':
        try:
            connection.dropDatabase()
        except OSError:
            pass
    open_db(connection.uri())
    init_db()


def teardown_module():
    connection = getConnection()
    if connection.dbName == 'sqlite':
        try:
            connection.dropDatabase()
        except OSError:
            pass


def load_inpx(inpx):
    import_inpx(os.path.join(os.path.dirname(__file__), inpx))



from fcntl import flock, LOCK_EX, LOCK_UN, LOCK_NB
from tempfile import gettempdir
import os
import socket


if os.access('/var/run/lock', os.W_OK):
    lock_dir = '/var/run/lock'
else:
    lock_dir = gettempdir()

if hasattr(os, 'getuid'):
    suffix = '-%d' % os.getuid()
else:
    suffix = ''

lock_fname = os.path.join(lock_dir, 'm_librarian%s.lock' % suffix)


def get_lock(port):
    try:
        lock_file = open(lock_fname, 'r')
    except IOError:  # no lock file
        pass
    else:
        try:
            flock(lock_file, LOCK_EX | LOCK_NB)
        except IOError:  # locked
            port = int(lock_file.readline())
            lock_file.close()
            return None, port
        else:
            flock(lock_file, LOCK_UN)
            lock_file.close()

    lock_file = open(lock_fname, 'w')
    lock_file.write(str(port))
    lock_file.close()
    lock_file = open(lock_fname, 'r')
    flock(lock_file, LOCK_EX | LOCK_NB)
    return lock_file, None


def close_lock(lock_file):
    flock(lock_file, LOCK_UN)
    lock_file.close()
    lock_file = open(lock_fname, 'w')
    lock_file.write('')
    lock_file.close()
    os.remove(lock_fname)


def get_open_port():
    # https://stackoverflow.com/a/2838309/7976758
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.bind(("", 0))
    # s.listen(1)
    port = s.getsockname()[1]
    s.close()
    return port



from sqlobject.sqlbuilder import AND, OR, func
from .config import get_config
from .db import Author, Book, Extension, Genre, Language

__all__ = [
    'mk_search_conditions',
    'search_authors', 'search_books', 'search_extensions',
    'search_genres', 'search_languages',
]


def _mk_search_conditions_with_operator(table, case_sensitive, comparison_op,
                                        values, expressions):
    if expressions is None:
        expressions = []
    _expressions = []
    for column, value in values.items():
        if column == 'id':
            _expressions.append(table.q.id == value)
            break
    if case_sensitive:
        for column, value in values.items():
            if column == 'id':
                continue
            _expressions.append(
                getattr(getattr(table.q, column), comparison_op)(value))
        for expr, value in expressions:
            _expressions.append(
                getattr(expr, comparison_op)(value))
    else:
        for column, value in values.items():
            if column == 'id':
                continue
            _expressions.append(
                getattr(func.lower(
                    getattr(table.q, column)),
                    comparison_op)(value.lower()))
        for expr, value in expressions:
            _expressions.append(
                getattr(func.lower(expr), comparison_op)(value.lower()))
    return _expressions


_comparison_operators = {
    'start': 'startswith',
    'substring': 'contains',
    'full': '__eq__',
}


def mk_search_conditions(table, search_type, case_sensitive, values,
                         expressions=None, join_expressions=None):
    if join_expressions is None:
        join_expressions = []
    return _mk_search_conditions_with_operator(
        table, case_sensitive, _comparison_operators[search_type],
        values, expressions) + join_expressions


def _search(table, search_type, case_sensitive, values,
            expressions=None, join_expressions=None, orderBy=None):
    conditions = mk_search_conditions(
        table, search_type, case_sensitive, values, expressions=expressions,
        join_expressions=join_expressions)
    return table.select(AND(*conditions), orderBy=orderBy)


def search_authors(search_type, case_sensitive, values,
                   expressions=None, orderBy=None):
    return _search(Author, search_type, case_sensitive, values,
                   expressions=expressions, orderBy=orderBy)


def search_books(search_type, case_sensitive, values, join_expressions=None,
                 orderBy=None, use_filters=False):
    if use_filters:
        config = get_config()
        lang_filter = config.getlist('filters', 'lang')
        deleted_filter = config.getint('filters', 'deleted')
        if lang_filter:
            if join_expressions is None:
                join_expressions = []
            lang_conditions = []
            for lang in lang_filter:
                lvalues = {'name': lang}
                conditions = mk_search_conditions(
                    Language, search_type, case_sensitive, lvalues)
                lang_conditions.append(conditions)
            join_expressions.append(Book.j.language)
            join_expressions.append(OR(*lang_conditions))
    conditions = mk_search_conditions(
        Book, search_type, case_sensitive, values,
        join_expressions=join_expressions)
    if use_filters and not deleted_filter:
        conditions.append(Book.q.deleted == False)  # noqa: E712
    return Book.select(AND(*conditions), orderBy=orderBy)


def search_extensions(search_type, case_sensitive, values, orderBy=None):
    return _search(Extension, search_type, case_sensitive, values,
                   orderBy=orderBy)


def search_genres(search_type, case_sensitive, values, orderBy=None):
    return _search(Genre, search_type, case_sensitive, values,
                   orderBy=orderBy)


def search_languages(search_type, case_sensitive, values, orderBy=None):
    return _search(Language, search_type, case_sensitive, values,
                   orderBy=orderBy)
